/*
 * Copyright (C) 1998-2004
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 * 
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.  
 */ 

#include "ltiObject.h"
#include "ltiMeanShiftSegmentationGenetics.h"
#include "ltiParetoFront.h"
#include "ltiStreamProgressInfo.h"

#include <iostream>
#include <fstream>

namespace lti {

  /**
   * The real class
   */
  class tester {
  public:
    void operator()(int argc,char *argv[]);
  };


  void tester::operator()(int argc,char *argv[]) {
    paretoFront::parameters pfPar;
    meanShiftSegmentationGenetics::parameters csPar;
    bool showProgress = true;
    int verboseLevel = 5;

    const char* file = "pareto_mean_shift.cfg";
 
    std::ifstream in(file);
    bool write=true;
    if (in) {
      lispStreamHandler lsh;
      lsh.use(in);
      write=!(lti::read(lsh,"showProgress",showProgress) &&
              lti::read(lsh,"verboseLevel",verboseLevel) &&
              lti::read(lsh,"pareto",pfPar) &&
              lti::read(lsh,"genetics",csPar));
    }
    if (write) {
      std::ofstream out(file);
      lispStreamHandler lsh;
      lsh.use(out);
      lti::write(lsh,"showProgress",showProgress);
      lti::write(lsh,"verboseLevel",verboseLevel);
      lti::write(lsh,"pareto",pfPar);
      lti::write(lsh,"genetics",csPar);

      std::cout << "Please edit file " << file 
                << " and ensure that the list of images is available."
                << std::endl;
      return;
    }

    // first a genetics object, with its corresponding parameters
    meanShiftSegmentationGenetics csg(csPar);
    // tell the parameters of the pareto front to use the given genetics object
    pfPar.setGeneticsObject(csg);
    // create a pareto object with the proper parameters
    paretoFront pf(pfPar);

    // we need to visualize progress
    streamProgressInfo progInfo;
    pf.setProgressObject(progInfo);
    pf.getProgressObject().setDetailLevel(verboseLevel);

    // and now, find the pareto front
    dmatrix front;
    pf.apply(front);
  }
}

int main(int argc,char *argv[]) {
  try {
    lti::tester testObj;
    testObj(argc,argv);
    std::cout << "Test at the end!" << std::endl;
  }
  catch (lti::exception& exp) {
    std::cout << "An LTI::EXCEPTION was thrown: ";
    std::cout << exp.what() << std::endl;
  }
  catch (std::exception& exp) {
    std::cout << "std::exception was thrown: ";
    std::cout << exp.what() << std::endl;
  }
  catch (...) {
    std::cout << "Unknown exception thrown!" << std::endl;
  }
  return 0;

} // main
