/*
 * Copyright (C) 2009 by Pablo Alvarado
 * 
 * This file is part of the lecture CE-5201 Digital Image Processing and
 * Analysis, at the Costa Rica Institute of Technology.
 *
 * The LTI-Lib-2 is free software; you can redistribute it and/or
 * modify it under the terms of the BSD License.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the authors nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _MINIMATRIX_H
#define _MINIMATRIX_H

#include <vector>

/**
 * Wrapper for a container
 */
template<class T>
class minimatrix {
private:
  /**
   * Data container
   */
  std::vector< std::vector< T > > data_;

  /**
   * Rows
   */
  int rows_;

  /**
   * Columns
   */
  int columns_;
  
public:
  /**
   * Contained type
   */
  typedef T value_type;

  /**
   * Constructor
   */
  minimatrix() : rows_(0), columns_(0) {
  }

  /**
   * Constructor
   */
  minimatrix(const int rows, const int cols) : rows_(0), columns_(0) {
    allocate(rows,cols);
  }

  /**
   * Allocate memory
   */
  void allocate(const int rows,
                const int columns) {
    rows_    = rows;
    columns_ = columns;

    data_.resize(rows);
    typename std::vector< std::vector< T > >::iterator it;
    const typename std::vector< std::vector< T > >::iterator eit = data_.end();
    for (it=data_.begin();it!=eit;++it) {
      (*it).resize(columns);
    }
  }

  int columns() const {
    return columns_;
  }

  int rows() const {
    return rows_;
  }

  /**
   * Access operator
   */
  T& at(int row,int col) {
    return data_[row][col];
  }

  /**
   * Access operator
   */
  const T& at(int row,int col) const {
    return data_[row][col];
  }
};

#endif
