/*
 * Copyright (C) 2009 by Pablo Alvarado
 * 
 * This file is part of the lecture CE-5201 Digital Image Processing and
 * Analysis, at the Costa Rica Institute of Technology.
 *
 * The LTI-Lib-2 is free software; you can redistribute it and/or
 * modify it under the terms of the BSD License.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the authors nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "progress.h"
#include "ltiRound.h"

#include <ostream>


/*
 * Constructor
 * 
 * Gives the total expected steps
 */
progress::progress(const int total) : total_(total),start_(0.0) {
}

void progress::start() {
  chronos_.start();
  start_ = chronos_.getTime();
}

/*
 * Print progress and estimated remaining time
 */
void progress::print(std::ostream& out,const int step) const {
  out.setf(std::ios::fixed,std::ios::floatfield);
  out.precision(2);
  out.width(8);
  out << lti::round(step*10000.0/total_)/100 << "%";
  if (step > 0) {
    double t  = (chronos_.getTime()-start_)/1000000.0; // elapsed time in s

    // estimated remaining time in seconds
    t *= (double(total_ - step - 1)/double(step));
    out << " \tERT: ";
    print(out,t);
  }
}

void progress::print(std::ostream& out,double t) const {
  const int days  = static_cast<int>(t/(60*60*24));
  t -= (days*(60*60*24));
  const int hours = static_cast<int>(t/(60*60));
  t -= (hours*60*60);
  const int mins  = static_cast<int>(t/60);
  t -= mins*60;
  const int secs  = lti::iround(t);
  
  if (days > 0) {
    out << days << "d " << hours << "h  ";
  } else if (hours > 0) {
    out << hours << "h " << mins << "m  " ;          
  } else if (mins > 0) {
    out << mins << "m " << secs << "s  ";
  } else if (secs > 0) {
    out << secs << "s  ";
  } else if (t>0.0) {
    out << lti::round(t*100.0)/100.0 << "s  ";
  }
}

void progress::stop(std::ostream& out) {
  double t  = (chronos_.getTime()-start_)/1000000.0;
  chronos_.stop();
  out << "                                              \r";
  out << "Total time: ";
  print(out,t);
  out << std::endl;
}
